 /*
 * Buggy mit IR-Fernsteuerung
 * 
 * Achtung: 
 *  ==> Beim Hochladen muss an der Fernsteuerung eine Taste gedrückt werden!
 * Anschluss IR-Empfangsdiode: 
 * - Daten: 5 (SDA) an I2C
 * - GND:   1 I2C
 * - Vcc:   2 I2C (5V)
 * 
 * Anschluss Motoren 
 * - linker Motor: MotorL (M2)
 * - rechter Motor: MotorR (M3)
 * 
 * Bibliotheken:
 * Arduino IR-Remote (v3.3.0)
 *  
 * Dirk Fox, 06.08.2016 Helmut Jawtusch 22.11.2021
 * Letzte Bearbeitung: 22.11.2021
 */
#include <adc_table.h>
#include <Ftduino.h>
#include <IRremote.h>

//#define DEBUG

#ifdef DEBUG
  #define Baudrate 115200
#endif

#define IRreceive 2

#define Arduino     0xFF
#define Vorwaerts   0x18E7 // Tastencodes der Fernsteuerung
#define Rueckwaerts 0x4AB5
#define Rechts      0x5AA5
#define Links       0x10EF
#define Schneller   0xB04F
#define Langsamer   0x6897
#define Stopp       0x38C7


#define MaxSpeed 64 // Maximalgeschwindigkeit der Motoren
#define Step 4 // Schrittweite der Geschwindigkeitsveraenderung

IRrecv irrecv(IRreceive);
uint16_t slow;
unsigned int mode = 2;
decode_results results;
byte motorspeed = 32;
void rechts(uint16_t speed)
  { mode = 1;
    ftduino.motor_set(Ftduino::M2, Ftduino::LEFT, speed );
    ftduino.motor_set(Ftduino::M3, Ftduino::RIGHT, speed ); 
    } 
void links(uint16_t speed)
  { mode = 2;
    ftduino.motor_set(Ftduino::M2, Ftduino::RIGHT, speed );
    ftduino.motor_set(Ftduino::M3, Ftduino::LEFT, speed );} 
void vorwaerts(uint16_t speed)
  { mode = 3;
    ftduino.motor_set(Ftduino::M2, Ftduino::RIGHT, speed);
    ftduino.motor_set(Ftduino::M3, Ftduino::RIGHT, speed);}   
void rueckwaerts(uint16_t speed)
  { mode = 4; 
    ftduino.motor_set(Ftduino::M2, Ftduino::LEFT, speed);
    ftduino.motor_set(Ftduino::M3, Ftduino::LEFT, speed);} 
void stopp(uint16_t speed)
  { ftduino.motor_set(Ftduino::M2, Ftduino::OFF, speed);
    ftduino.motor_set(Ftduino::M3, Ftduino::OFF, speed);} 
void set (uint16_t m) {
  switch(m) {
    case 1: rechts(motorspeed);
      break; 
    case 2: links(motorspeed);
      break;
    case 3: vorwaerts(motorspeed);
      break; 
    case 4: rueckwaerts(motorspeed);
      break; 
} }        
          
void buggycontrol(unsigned int value) { // Steuerung des Buggy
  switch(value) {
    case Vorwaerts: vorwaerts(motorspeed);
#ifdef DEBUG
      Serial.println("Vorwärts");
#endif
      break;
    case Rueckwaerts: rueckwaerts(motorspeed);
#ifdef DEBUG
      Serial.println("Rückwärts");
#endif
      break;
    case Rechts: rechts(motorspeed);
#ifdef DEBUG
      Serial.println("Nach rechts");
#endif
      break;
    case Links: links(motorspeed);
#ifdef DEBUG
      Serial.println("Nach links");
#endif
      break;
    case Schneller:
      if (motorspeed <= MaxSpeed-Step)
        {motorspeed += Step; set(mode);}
#ifdef DEBUG
      Serial.print("Schneller (");
      Serial.print(motorspeed);
      Serial.println(")");
#endif
    break;
    case Langsamer:
      if (motorspeed >= 24)
        {motorspeed -= Step; set(mode);}
#ifdef DEBUG
      Serial.print("Langsamer (");
      Serial.print(motorspeed);
      Serial.println(")");
#endif
    break;
    case Stopp: stopp(motorspeed);
#ifdef DEBUG
      Serial.println("Stopp");
#endif
      break;
  } 
}

void setup() {
    ftduino.init();  
#ifdef DEBUG
  Serial.begin(Baudrate); // seriellen Monitor starten
  while(!Serial); 
#endif
  irrecv.enableIRIn(); // IR-Empfänger starten
#ifdef DEBUG
  Serial.println("Infrarot-Fernsteuerung");
#endif
}
void loop() {
  while (!irrecv.decode(&results)); // warte bis Wert empfangen
  if ((results.value >> 16) == Arduino) {
#ifdef DEBUG
  Serial.print("Taste: ");
  Serial.println(results.value, HEX);
#endif    
    buggycontrol(results.value & 0xFFFF); // Ansteuerung des Buggy
  }  
  irrecv.resume(); // nächsten Wert empfangen
}
